#include <iostream>
#include <map>
#include <vector>

void print(const auto& vec) {
	if (vec.empty()) {
		std::cout << "{}\n";
		return;
	}
	auto sep = "{ ";
	for (auto& x : vec) {
		std::cout << sep << x;
		sep = ", ";
	}
	std::cout << " }\n";
}

auto filter(const auto& cont, auto pred) {
	using T = std::decay_t<decltype(*cont.begin())>;
	std::vector<T> result;
	for (auto& x : cont)
		if (pred(x))
			result.push_back(x);
	return result;
}

auto less_than(auto value) {
	return [=](auto num) { return num < value; };
}

int main() {
	std::vector v = { 1, 2, 3, 4, 5, 6, 7, 8, 9 };

	auto v1 = filter(v,
		[](int num) { return num % 2 == 0; });

	auto v2 = filter(v, less_than(5));

	auto v3 = filter(v,
		[i = 0](int num) mutable {
			return num == i++;
		});

	print(v1);
	print(v2);
	print(v3);
}
