#include <iostream>

struct reporter {
	reporter() { std::cout << "constructed\n"; }
	~reporter() { std::cout << "destroyed\n"; }
	reporter(const reporter&) { std::cout << "copied\n"; }
	reporter(reporter&&) { std::cout << "moved\n"; }
	reporter& operator=(const reporter&) {
		std::cout << "copy-assigned\n";
		return *this;
	}
	reporter& operator=(reporter&&) {
		std::cout << "move-assigned\n";
		return *this;
	}
};

struct example {
	reporter r;
	int value = 0;

	void cmethod() const {
		std::cout << "const method\n";
	}

	void method() {
		std::cout << "non-const method\n";
	}

	auto lambda_this() {
		// [&] has the same effect
		return [this] {
			cmethod();
			method();
			std::cout << value << '\n';
		};
	}

	auto lambda_star_this() {
		return [*this] {
			cmethod();
			// method();
			std::cout << value << '\n';
		};
	}

	auto lambda_star_this_mut() {
		return [*this]() mutable {
			cmethod();
			method();
			std::cout << value << '\n';
		};
	}
};

int main() {
	example ex;
	auto fun1 = ex.lambda_this();
	ex.value = 7;
	fun1();
	std::cout << "---\n";
	auto fun2 = ex.lambda_star_this();
	ex.value = 17;
	fun2();
	std::cout << "---\n";
	auto fun3 = ex.lambda_star_this_mut();
	ex.value = 27;
	fun3();
	std::cout << "---\n";
}
